# canonicalize_radical : replaces sqrt(f(x)^2) by +|f(x)| or -|f(x)|
# the sign is the sign of f(x) when x->infinity : this can cause trouble, be careful

########################
# Some useful routines #
########################

# list of discs (x,y,r) -> svg file "name"
def tosvg(l,name):
    f=open(name,'w')
    f.write('<svg>\n')
    for (x,y,t) in l:
        f.write(' <circle cx="%fmm" cy="%fmm" r="%fmm" fill="%s" stroke="black" />\n'%(50*x,50*y,50*t,'#5599ff' if t<1 else '#ffd42a'))
    f.write('</svg>\n')
    f.close()

# INPUT: discs (x1,y1,r1), (x2,y2,r2) and radius r3
# OUTPUT: disc (x3,y3,r3) tangent to both other discs
def stick((x1,y1,r1),(x2,y2,r2),r3,direct=true):
    l=sqrt((x2-x1)^2+(y2-y1)^2) # l=r1+r2 if discs 1 & 2 are tangents (not mandatory)
    cosa=((r1+r3)^2+l^2-(r2+r3)^2)/(2*(r1+r3)*l)
    sina=(1 if direct else -1)*sqrt(1-cosa^2)
    x3=x1+(cosa*(x2-x1)-sina*(y2-y1))*(r1+r3)/l
    y3=y1+(sina*(x2-x1)+cosa*(y2-y1))*(r1+r3)/l
    return (x3.canonicalize_radical(),y3.canonicalize_radical(),r3)

# INPUT: a radical expression in r (symbolic expression)
# OUTPUT: a radicande of g
def choose_sqrt_in(g):
    (x,y,z)=(SR.wild(0),SR.wild(1),SR.wild(2))
    s=0
    while true:
        m=g.match(x*sqrt(y)+z)
        if m==None: # sqrt(f) is ignored if written f^(1/2)..
            return s
        if len(str(m[y]))>len(str(s)):
            s=m[y]
        g=m[x]+m[z]

# find the algebraic roots in [a,b] of a radical expression f
def find_roots(f,a,b):
    f=numerator(f.canonicalize_radical())
    g=expand(f)
    r=g.variables()[0]
    while not g.is_polynomial(r):
        y=choose_sqrt_in(g) # choose a sqrt to be removed
        x=g.coefficient(sqrt(y))
        z=g-x*sqrt(y) # g=x*sqrt(y)+z
        g=x^2*y-z^2
        g=g.canonicalize_radical()
    g=AA[r](g) # QQ
    return [i for i in g.roots(AA,multiplicities=false) if i>=a and i<=b and RIF(f.subs(r=i)).contains_zero()]

var('r')

# list ((a,b),d) where d(r) lower bound for the density for r in [a,b]
densities=[(RIF(0,1),(1/sqrt(12)).function(r))]
density=lambda r: pi*max([z[1](r) for z in densities if r in z[0]])

#####################################
# Flow 0 <-> r8 <-> r4 <-> r1 <-> 1 #
#####################################

# a horizontal red line shows the density of the hexagonal compact packing
flow841=line([(0,pi/sqrt(12)),(1,pi/sqrt(12))],color='red')

# r1 <-> 1
d0=(0,0,r)
d1=(2*r,0,r)
d2=stick(d0,d1,1)
d3=stick(d2,d1,1)
# r1 : when d3[1]=1
r1=find_roots(d3[1]-1,0,1)[0]
# density
d=((1+r^2)/((d3[0]-r)*(d2[1]+d3[1])))
d=d.canonicalize_radical()
flow841+=plot(pi*d(r),r,r1,1)
# ratio for which density of hcp is reached
a5=find_roots(d-1/sqrt(12),r1,1)[0]
flow841+=line([(a5,pi/sqrt(12)),(a5,0.9)],color='black')
densities.append((RIF(r1,1),d.function(r)))

# r4 <-> r1
d0=(0,0,1)
d1=(2,0,1)
d2=stick(d0,d1,r)
d3=stick(d2,d1,1)
d4=stick(d2,d3,1)
# r4 : when d4[1]=2
r4=find_roots(d4[1]-2,0,1)[0]
# density
d=((1+r^2)/((d1[0]*d3[1]-d1[1]*d3[0])/2+(d3[0]*d4[1]-d3[1]*d4[0])/2))
d=d.canonicalize_radical()
flow841+=plot(pi*d(r),r,r4,r1)
# ratio for which density of hcp is reached
[a1,a4]=find_roots(d-1/sqrt(12),r4,r1)
flow841+=line([(a1,pi/sqrt(12)),(a1,0.9)],color='black')
flow841+=line([(a4,pi/sqrt(12)),(a4,0.9)],color='black')
densities.append((RIF(r4,r1),d.function(r)))

# r8 <-> r4
d0=(0,0,1)
d1=(2,0,1)
d2=stick(d0,d1,r)
d3=stick(d0,d2,1)
# r8 : when d3[0]=1
r8=find_roots(d3[0]-1,0,1)[0]
# density
d=(1+r^2)/(d1[0]*d3[1])
d=d.canonicalize_radical()
flow841+=plot(pi*d(r),r,r8,r4)
# ratio for which density of hcp is reached
l=find_roots(d-1/sqrt(12),r8,r4)
for i in l:
    flow841+=line([(i,pi/sqrt(12)),(i,0.9)],color='black')
densities.append((RIF(r8,r4),d.function(r)))

# 0 <-> r8
d=(1+r^2)/(2*sqrt(3))
flow841+=plot(pi*d(r),r,0,r8)
densities.append((RIF(0,r8),d.function(r)))

#####################################
# Flow 0 <-> r8 <-> r7 <-> r3 <-> 1 #
#####################################

flow873=line([(0,pi/sqrt(12)),(1,pi/sqrt(12))],color='red')

# r3 <-> 1
d0=(0,0,1)
d1=(1+r,0,r)
d2=stick(d0,d1,r)
d3=stick(d2,d1,1)
d4=stick(d2,d3,r)
d5=stick(d2,d4,1)
# r3 : when d3[1]=1
r3=find_roots(d3[1]-1,0,1)[0]
# density
d=(1+2*r^2)/(d3[0]*d5[1]-d3[1]*d5[0])
d=d.canonicalize_radical()
flow873+=plot(pi*d(r),r,r3,1)
# ratio for which density of hcp is reached
l=find_roots(d-1/sqrt(12),r3,1)
for i in l:
    flow873+=line([(i,pi/sqrt(12)),(i,0.9)],color='black')
densities.append((RIF(r3,1),d.function(r)))


# r7 <-> r3
d0=(0,0,1)
d1=(2,0,1)
d2=stick(d0,d1,r)
d3=stick(d0,d2,r)
d4=stick(d3,d2,1)
# r7: when d1 and d4 in contact
r7=find_roots((d1[0]-d4[0])^2+(d1[1]-d4[1])^2-4,0,1)[0]
# density
d=(1+2*r^2)/(d1[0]*d4[1]-d1[1]*d4[0])
d=d.canonicalize_radical()
flow873+=plot(pi*d(r),r,r7,r3)
# ratio for which density of hcp is reached
l=find_roots(d-1/sqrt(12),r7,r3)
for i in l:
    flow873+=line([(i,pi/sqrt(12)),(i,0.9)],color='black')
a2=l[1]
densities.append((RIF(r7,r3),d.function(r)))

# r8 <-> r7
d0=(0,0,1)
d1=(2,0,1)
d2=stick(d0,d1,r)
d3=stick(d0,d2,1)
# r8: when d1 touches d3, i.e., d3[0]=1
r8=find_roots(d3[0]-1,0,1)[0]
# density
d=(1+2*r^2)/(d1[0]*d3[1]-d1[1]*d3[0])
d=d.canonicalize_radical()
flow873+=plot(pi*d(r),r,r8,r7)
densities.append((RIF(r8,r7),d.function(r)))

# 0 <-> r8
d=(1+2*r^2)/(2*sqrt(3))
flow873+=plot(pi*d(r),r,0,r8)
densities.append((RIF(0,r8),d.function(r)))

##############################
# Flow 0 <-> r9 <-> r6 <-> 1 #
##############################

flow96=line([(0,pi/sqrt(12)),(1,pi/sqrt(12))],color='red')

# r6 <-> 1
# the packing cannot be described by sticking discs one after the other
# we use another method, more algebraic, maybe more nice but quite slow when there are many discs
K.<x1,y1,x2,y2,x3,y3,x4,y4,x5,y5,r,d>=QQ[]
# equations on disc coordinates
eqs=[
y1,
x1-2*x2,
x1+x2-(x3+x4+x5),
y1+y2-(y3+y4+y5),
x3^2+y3^2-(1+r)^2,
(x4-x3)^2+(y4-y3)^2-(r+r)^2,
(x4-x5)^2+(y4-y5)^2-(r+r)^2,
(x3-x5)^2+(y3-y5)^2-(r+r)^2,
(x4-x1)^2+(y4-y1)^2-(1+r)^2,
(x2-x5)^2+(y2-y5)^2-(1+r)^2,
(x1-2*x3)^2+(y3+y3)^2-(r+r)^2, # contact between d3 and its symmetric along X axes
d*x1*y2-(1+6*r^2)
]
# r6 : when y3=r
r6=ideal(K,eqs+[y3-r]).variety(AA)[4][r]
# equation in d and r
I=ideal(K,eqs)
J=I.elimination_ideal([x1,y1,x2,y2,x3,y3,x4,y4,x5,y5])
P=QQ[r,d](J.gens()[0])
P=P//(12*d^2-28*r-77) # remove "bad" factor (no suitable roots)
# density
d=SR(P).solve(SR(d))[3].right_hand_side().canonicalize_radical()
flow96+=plot(d(r)*pi,r,r6,1)
# ratio for which density of hcp is reached
l=find_roots(d-1/sqrt(12),r6,1)
for i in l:
    flow96+=line([(i,pi/sqrt(12)),(i,0.9)],color='black')
var('r')
densities.append((RIF(r6,1),d.function(r)))

# r9 <-> r6
d0=(0,0,1)
d1=(1+r,0,r)
d2=stick(d0,d1,r)
d3=stick(d2,d1,r)
d4=stick(d3,d1,1)
d5=stick(d2,d3,1)
# r9: when d0 touches d4
r9=find_roots((d0[0]-d4[0])^2+(d0[1]-d4[1])^2-4,0,1)[0]
# density
d=(1+6*r^2)/(d4[0]*d5[1]-d4[1]*d5[0])
d=d.canonicalize_radical()
flow96+=plot(pi*d(r),r,r9,r6)
# ratio for which density of hcp is reached
l=find_roots(d-1/sqrt(12),r9,r6)
for i in l:
    flow96+=line([(i,pi/sqrt(12)),(i,0.9)],color='black')
densities.append((RIF(r9,r6),d.function(r)))

# 0 <-> r9
d=(1+6*r^2)/(2*sqrt(3))
flow96+=plot(pi*d(r),r,0,r9)
densities.append((RIF(0,r9),d.function(r)))

########################
# Flow r9 <-> r2 <-> 1 #
########################

flow2=line([(0,pi/sqrt(12)),(1,pi/sqrt(12))],color='red')

# r2 <-> 1
d0=(0,0,1)
d1=(2,0,1)
d2=stick(d0,d1,1)
d3=stick(d2,d1,r)
d4=stick(d3,d1,r)
d5=stick(d3,d4,r)
d6=stick(d5,d4,1)
d7=stick(d3,d5,1)
# r2: when d2 touches d7
r2=find_roots((d2[0]-d7[0])^2+(d2[1]-d7[1])^2-4,0,1)[0]
# density
d=(3+3*r^2)/((d6[0]-d2[0])*d7[1]-(d6[1]-d2[1])*d7[0])
d=d.canonicalize_radical()
flow2+=plot(d(r)*pi,r,r2,1)
# ratio for which density of hcp is reached
a3=find_roots(d-1/sqrt(12),r2,1)[0]
flow2+=line([(a2,pi/sqrt(12)),(a2,0.9)],color='black')
densities.append((RIF(r2,1),d.function(r)))

# r9 <-> r2
d0=(0,0,1)
d1=(1+r,0,r)
d2=stick(d0,d1,r)
d3=stick(d2,d1,r)
d4=stick(d2,d3,1)
d5=stick(d3,d1,1)
d6=stick(d4,d5,1)
d7=stick(d0,d4,1)
# density
d=(3+3*r^2)/(d6[0]*(d7[1]-d5[1])-d6[1]*(d7[0]-d5[0]))
d=d.canonicalize_radical()
flow2+=plot(d(r)*pi,r,r9,r2)
# ratio for which density of hcp is reached
l=find_roots(d-1/sqrt(12),r9,r2)
for i in l:
    flow2+=line([(i,pi/sqrt(12)),(i,0.9)],color='black')
densities.append((RIF(r9,r2),d.function(r)))

# 0 <-> r9
d=(1+r^2)/(2*sqrt(3))
flow2+=plot(pi*d(r),r,0,r9)
densities.append((RIF(0,r9),d.function(r)))

########################
# Flow rx <-> r5 <-> 1 #
########################

flow5=line([(0,pi/sqrt(12)),(1,pi/sqrt(12))],color='red')

# r5 <-> 1
d0=(0,0,r)
d1=(2*r,0,r)
d2=stick(d0,d1,r)
d3=stick(d0,d2,r)
d4=stick(d2,d1,1)
d5=stick(d3,d2,1)
# r5: when d4[1]=1
r5=find_roots(d4[1]-1,0,1)[0]
# density
d=(2+7*r^2)/(2*d4[0]*(d4[1]+d5[1]))
d=d.canonicalize_radical()
flow5+=plot(d(r)*pi,r,r5,1)
# ratio for which density of hcp is reached
l=find_roots(d-1/sqrt(12),r5,1)
for i in l:
    flow5+=line([(i,pi/sqrt(12)),(i,0.9)],color='black')
densities.append((RIF(r5,1),d.function(r)))

#rz=find_roots(diff(d,r),r5,1)
#tosvg([[i.subs(r=rz) for i in d] for d in [d0,d1,d2,d3,d4]],"out.svg")


# rx <-> r5 (same pb as for r6 <-> 1: no construction by sticking discs one by one)
K.<x1,y1,x2,y2,x3,y3,x4,y4,r,d>=QQ[]
eqs=[
y1,y3-1,x4,2*x2-x1,
x2^2+y2^2-4*r^2,
(x1-x2)^2+(y1-y2)^2-(r+r)^2,
(x1-x3)^2+(y1-y3)^2-(1+r)^2,
(x2-x4)^2+(y2-y4)^2-(1+r)^2,
(x3-x4)^2+(y3-y4)^2-(1+1)^2,
d*x3*(y3+y4)-(1+7/2*r^2)
]
I=ideal(K,eqs)
I.dimension()
J=I.elimination_ideal([x1,y1,x2,y2,x3,y3,x4,y4])
P=QQ[r][d](J.gens()[0]) # polynomial has degree 6 in d^2 -> no closed expression :(
# rx: when y2=r
rx=ideal(K,eqs+[y2-r]).variety(AA)[4][r]
# drawing : implicit plot is quite long ;
#f=lambda r,d: P(r,RR(d/pi))
#flow5+=implicit_plot(f,(rx,r5),(0,1)) # 19s
# drawing with find_root: raises "NotImplementedError: Brent's method failed to find a zero for f on the interval" for some values -> pedestrian way
seg = lambda a,b,k: [RDF(a+i*(b-a)/k) for i in range(k+1)]
l=[]
for i in seg(rx,r5,300):
    try:
        l.append((i,find_root(SR(P.subs(r=i)),0,1/pi)*pi))
    except:
        print i
flow5+=line(l)

# ratio for which density of hcp is reached
Q=QQ[r](P.subs(d=1/sqrt(12)))
l=Q.roots(AA)[1]
for i in l:
    flow5+=line([(i,pi/sqrt(12)),(i,0.9)],color='black')

# different lower bound, not that good but easier: shrink the small discs, keep contact with two diametally opposed large discs, the other large discs form a hexagon
var('r')
y=sqrt(3)*r + sqrt(2*r + 1)
x=sqrt(4-(y-1)^2)
d=(2+7*r^2)/(x*(1+y)*2)
densities.append((RIF(0,r5),d.function(r)))


######################################
# Flow r9 <-> rc <-> rb <-> ra <-> 1 #
######################################

flow9cba=line([(0,pi/sqrt(12)),(1,pi/sqrt(12))],color='red')

# ra <-> 1
d0=(0,0,1)
d1=(1+r,0,r)
d2=stick(d0,d1,r)
d3=stick(d0,d2,r)
d4=stick((-1-r,0,r),d3,r,direct=false) # indirect to avoid sign problme with canonicalize_radical
d5=stick(d2,d1,1)
# ra: when d4 touches d0
ra=find_roots(d4[0]^2+d4[1]^2-(1+r)^2,0,1)[0]
# density (less than hcp over [ra,1])
d=(1+4*r^2)/(d5[0]*(d4[1]+d3[1])-d5[1]*(d4[0]+d3[0]))
d=d.canonicalize_radical()
flow9cba+=plot(d(r)*pi,r,ra,1)
densities.append((RIF(ra,1),d.function(r)))

# rb <-> ra
d0=(0,0,1)
d1=(1+r,0,r)
#d1=(sqrt(1+2*r),-r,r)
d2=stick(d0,d1,r)
d3=stick(d2,d1,1)
d4=stick(d0,d2,r)
d5=stick(d2,d3,r)
d6=stick(d4,d5,r,direct=false) # indirect to avoid sign problme with canonicalize_radical
d7=stick(d4,d6,1)
# rb: when d7 touches d0
rb=find_roots(d7[0]^2+d7[1]^2-4,0,1)[0]
# ra: when d4 touches d5-d3
#ra=find_roots((d4[0]-d5[0]+d3[0])^2+(d4[1]-d5[1]+d3[1])^2-4*r^2,0,1)[0]
# density
d=(1+4*r^2)/(d3[0]*d7[1]-d3[1]*d7[0])
d=d.canonicalize_radical()
flow9cba+=plot(d(r)*pi,r,rb,ra)
# ratio for which density of hcp is reached
l=find_roots(d-1/sqrt(12),rb,ra)
for i in l:
    flow9cba+=line([(i,pi/sqrt(12)),(i,0.9)],color='black')
densities.append((RIF(rb,ra),d.function(r)))

# rc <-> rb
d0=(0,0,1)
d1=(2,0,1)
d2=stick(d0,d1,r)
d3=stick(d2,d1,r)
d4=stick(d0,d2,r)
d5=stick(d4,d3,r)
d6=stick(d4,d5,1)
# rc: when d6 touches d0
rc=find_roots(d6[0]^2+d6[1]^2-4,0,1)[0]
# density
d=(1+4*r^2)/(d1[0]*d6[1])
d=d.canonicalize_radical()
flow9cba+=plot(d(r)*pi,r,rc,rb)
# ratio for which density of hcp is reached
l=find_roots(d-1/sqrt(12),rc,rb)
for i in l:
    flow9cba+=line([(i,pi/sqrt(12)),(i,0.9)],color='black')
densities.append((RIF(rc,rb),d.function(r)))

# r9 <-> rc
d0=(0,0,1)
d1=(2,0,1)
d2=stick(d0,d1,r)
d3=stick(d0,d2,r)
d4=stick(d0,d3,1)
# r9: when d1 touches d4, i.e. d4[0]=1
r9=find_roots(d4[0]-1,0,1)[0]
# density
d=(1+4*r^2)/(d1[0]*d4[1])
d=d.canonicalize_radical()
flow9cba+=plot(d(r)*pi,r,r9,rc)
# ratio for which density of hcp is reached
l=find_roots(d-1/sqrt(12),r9,rc)
for i in l:
    flow9cba+=line([(i,pi/sqrt(12)),(i,0.9)],color='black')
densities.append((RIF(r9,rc),d.function(r)))

# 0 <-> r9
d=(1+4*r^2)/(2*sqrt(3))
flow9cba+=plot(pi*d(r),r,0,r9)
densities.append((RIF(0,r9),d.function(r)))


#########################
# Flow r9 <-> rx <-> ry #
#########################
# very small improvement around rx=0.1666..., not discussed in the paper since does not impact the main theorem.

flow9xy=line([(0,pi/sqrt(12)),(1,pi/sqrt(12))],color='red')

# rx <-> ry
d0=(0,0,r)
d1=(-r,sqrt(3)*r,r)
d2=(r,sqrt(3)*r,r)
d4=stick(d1,d2,1)
d3=(r,-sqrt(3)*r,r)
d5=stick(d2,d3,1)
# density
d=(1+5*r^2)/(d5[0]*d4[1]*2)
d=d.canonicalize_radical()
# rx: when d4 touches d5
rx=find_roots((d4[0]-d5[0])^2+(d4[1]-d5[1])^2-4,0,1)[0]
# ry: when d2 touches d6
ry=find_roots((d2[0]+d2[0]-d5[0]-d4[0])^2+(d2[1]+d2[1]-d4[1]+d5[1])^2-(r+r)^2,0,1)[0]
#d6=stick(d4,d5,r)
#ry=find_roots((d2[0]-d6[0])^2+(d2[1]-d6[1])^2-(r+r)^2,0,1)[0]
flow9xy+=plot(d(r)*pi,r,rx,ry)
# ratio for which density of hcp is reached
l=find_roots(d-1/sqrt(12),rx,ry)
for i in l:
    flow9xy+=line([(i,pi/sqrt(12)),(i,0.9)],color='black')
densities.append((RIF(rx,ry),d.function(r)))

# r9 <-> rx
d0=(0,0,1)
d1=(2,0,1)
d2=stick(d0,d1,r)
d3=stick(d2,d1,r)
d4=stick(d3,d1,1)
d=(1+5*r^2)/(2*d4[1])
flow9xy+=plot(d(r)*pi,r,r9,rx)
densities.append((RIF(r9,rx),d.function(r)))

#################
# interstitials #
#################


# simply take all the small circles centered on the triangular lattice which lie inside the hole between three large

def interstitial(r,draw=false):
    l=[]
    s3=RR(sqrt(3))
    for i in range(-ceil(1/4/r),ceil(1/4/r)):
        for j in range(-ceil(1/3/r),ceil(1/6/r)):            
            x=(i+j/2)*2*r
            y=j*s3*r
            if (x+1)^2+(y+1/s3)^2<(1+r)^2 or x^2+(y-s3+1/s3)^2<(1+r)^2 or (x-1)^2+(y+1/s3)^2<(1+r)^2:
                continue
            l.append([x,y])
    d=RIF(pi*(1/2+len(l)*r^2)/s3)
    if draw:
        g=circle([-1,-1/s3],1)+circle([1,-1/s3],1)+circle([0,s3-1/s3],1)+sum([circle([x,y],r) for [x,y] in l])
        g.show(axes=False);    
    return d

# approximation (1min)
z=[(i,interstitial(i)) for i in seg(0,r9,1000)[10:]]
flow0=line(z)

#################
# All the flows #
#################

flows=flow0+flow841+flow873+flow96+flow2+flow5+flow9cba #+flow9xy


##################################
# upper bounds Florian and Blind #
##################################

# jonction Florian/blind
rb1=0.6735210083310348

df=lambda r: RIF((arcsin(r/(1+r))*(1-r^2)+pi/2*r^2)/(r*sqrt(1+2*r)))

florian=line([(x,d(x)) for x in seg(1e-5,1,100)],color='red')

# Blind rejoint hcp
rb2=RIF(sqrt((7*tan(pi/7)-sqrt(12))/(sqrt(12)-5*tan(pi/5))))

# rb2^2 racine de
Q=378890468381881*x^24 - 45838704100445856*x^23 - 11432729996426105652*x^22 + 1036725093604542790368*x^21 + 8434790797035805242522*x^20 - 1168530028685510204114016*x^19 - 2224797758035419710146900*x^18 + 34801237914169933839246624*x^17 + 44886751790096923526412719*x^16 - 307994270351521337362778688*x^15 - 335603809021931400840186152*x^14 + 1061612899380314246064341184*x^13 + 903053454160788953614263884*x^12 - 1717676358164370500429460672*x^11 - 968853666616129122003708040*x^10 + 1348594838404478974841844288*x^9 + 441830047361033609105418023*x^8 - 498790022575892442207010080*x^7 - 87136686533908198907783716*x^6 + 79943432560876134158286432*x^5 + 6716669613456319424967866*x^4 - 4260457241890905046626528*x^3 - 44701039736454656151460*x^2 + 325640944030907723424*x + 2180902918040617681


db=lambda r: RIF(pi*(1+r^2)/(7*tan(pi/7)+r^2*5*tan(pi/5)))

blind=line([(x,d(x)) for x in seg(rb1,rb2,100)],color='red')

###############################################
# peak of the mysterious island around r=0.48 #
###############################################

# compute r which allows the 1-corona [0,2,7]
d0=(0,0,1)
d1=(2,0,1)
d2=stick(d0,d1,r)
d3=stick(d0,d2,r)
d4=stick(d0,d3,r)
d5=stick(d0,d4,r)
d6=stick(d0,d5,r)
d7=stick(d0,d6,r)
d8=stick(d0,d7,r)
d9=stick(d0,d8,r)
D1=stick(d0,d9,1)
rm=find_roots(d1[0]-D1[0],0.4,0.5)[0]

